"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = require("react");
var is_plain_object_1 = __importDefault(require("is-plain-object"));
var SimpleStore_1 = __importDefault(require("./SimpleStore"));
var defaultStoreConfig = {
    name: 'DefaultStore',
    debug: false
};
var devToolsConect = (window &&
    // @ts-ignore
    window.__REDUX_DEVTOOLS_EXTENSION__ &&
    // @ts-ignore
    window.__REDUX_DEVTOOLS_EXTENSION__.connect) || (function () { });
/**
 * Create new Store for using react hook
 *
 * @param {Object} initialState Start state
 * @param {Object} actions Object with actions
 * @param {Object} config Object with addidional configuration
 *
 * @retrun store, dispatch function and useStore hook
 */
var initStore = function (initialState, actions, config) {
    if (config === void 0) { config = defaultStoreConfig; }
    var store = new SimpleStore_1.default(initialState);
    var devTools = undefined;
    if (config.debug) {
        devTools = devToolsConect({
            name: config.name,
            features: {
                jump: true
            }
        });
        devTools.init(store.getState());
        devTools.subscribe(function (message) {
            if (message.type === 'DISPATCH' && message.state) {
                store.setState(JSON.parse(message.state));
            }
        });
    }
    var dispatchAction = function (actionFunction, payload) {
        var actionResult = actionFunction({
            state: store.getState(),
            getState: store.getState,
            setState: store.setState,
            dispatch: dispatchAction,
        }, payload);
        if (is_plain_object_1.default(actionResult)) {
            store.setState(actionResult);
        }
        if (devTools) {
            devTools.send(actionFunction.name, store.getState());
        }
    };
    var memoActions = Object.entries(actions || {})
        .reduce(function (result, _a) {
        var key = _a[0], action = _a[1];
        if (typeof action === 'function') {
            result[key] = function (payload) {
                return new Promise(function (resolve) {
                    requestAnimationFrame(function () {
                        dispatchAction(action, payload);
                        resolve(store.getState());
                    });
                });
            };
        }
        return result;
    }, {});
    var useStore = function () {
        var usingProps = react_1.useMemo(function () { return new Set; }, []);
        var _a = react_1.useState({}), innerState = _a[0], setInnerState = _a[1];
        var proxyObject = react_1.useMemo(function () { return new Proxy({}, {
            get: function (_, prop) {
                usingProps.add(prop);
                var outerState = store.getState();
                if (prop in outerState) {
                    return outerState[prop];
                }
                if (prop in memoActions) {
                    return memoActions[prop];
                }
                return undefined;
            },
            set: function () {
                throw new ReferenceError('You can\'t modify state directly');
            }
        }); }, []);
        react_1.useEffect(function () {
            return store.subscribe(function (newState) {
                var isPropsChanged = false;
                var newInnerState = {};
                usingProps.forEach(function (key) {
                    if (key in memoActions) {
                        return;
                    }
                    isPropsChanged = isPropsChanged || !Object.is(newState[key], innerState[key]);
                    newInnerState[key] = newState[key];
                });
                if (isPropsChanged) {
                    setInnerState(newInnerState);
                }
            });
        }, [innerState]);
        return proxyObject;
    };
    return {
        useStore: useStore,
        store: store,
        dispatch: dispatchAction
    };
};
exports.default = initStore;
